/* hacktest.c - Test HackRf / (c) 2015 Martin Albert <ma@debian.org> */

#include <stdio.h>
#include <libhackrf/hackrf.h>

int debug = 0, verbose = 0;
unsigned long count = 0;

enum hackrf_error res;

hackrf_device *dev = NULL;
uint8_t hackrf_id;
const char *hackrf_name;
read_partid_serialno_t hackrf_no;
char hackrf_version[20];

int tr(enum hackrf_error res, char *fn) {
  if (debug)
    fprintf(stderr, "%s=%s\n", hackrf_error_name(res), fn);
  return res != HACKRF_SUCCESS;
}

int hackrf_info(int p) {
  res = hackrf_board_id_read(dev, &hackrf_id);
  if (tr(res, "hackrf_board_id_read"))
    return 0;
  hackrf_name = hackrf_board_id_name((enum hackrf_board_id) hackrf_id);
  res = hackrf_version_string_read(dev, hackrf_version, (uint8_t)sizeof(hackrf_version));
  if (tr(res, "hackrf_version_string_read"))
    return 0;
  res = hackrf_board_partid_serialno_read(dev, &hackrf_no);
  if (tr(res, "hackrf_board_partid_serialno_read"))
    return 0;

  if (p) {
  FILE *o = stderr; if (p > 1) o = stdout;
  fprintf(o,
    "HackRf v%d %s, Version: %s\nSerialNo: %x.%x-%x.%x.%x.%x\n",
    hackrf_id, hackrf_name, hackrf_version,
    hackrf_no.part_id[0], hackrf_no.part_id[1],
    hackrf_no.serial_no[0], hackrf_no.serial_no[1],
    hackrf_no.serial_no[2], hackrf_no.serial_no[3]
  ); }
  return 1;
}

int hackrf_begin() {
  res = hackrf_init();
  if (tr(res, "hackrf_init"))
    return 0;

  res = hackrf_open(&dev);
  if (tr(res, "hackrf_open"))
    return 0;

  return hackrf_info(debug*2+verbose);
}

void hackrf_end() {
  if (dev)
    tr(res = hackrf_close(dev), "hackrf_close");
  tr(res = hackrf_exit(), "hackrf_exit");
}

int hackrf_hush() {
  res = hackrf_stop_tx(dev);
  res = hackrf_stop_rx(dev);
  res = hackrf_set_antenna_enable(dev, 0);
  res = hackrf_set_amp_enable(dev, 0);
  res = hackrf_set_txvga_gain(dev, 0);
  res = hackrf_set_vga_gain(dev, 0);
  res = hackrf_set_lna_gain(dev, 0);
  /* tune low freq and set hipass - 2014.8.1 */
  res = hackrf_set_freq_explicit(dev, (uint64_t)2.424e9,
    (uint64_t)2.424e9, RF_PATH_FILTER_HIGH_PASS);
  tr(res, "hackrf_set_freq_explicit");
  return 0;
}


/* blink(3) */
int  _dummy() {return -1;}
void blinko(int times) {
  while (times--) {
    res = hackrf_start_rx(dev, _dummy, NULL);
    sleep(1);
    res = hackrf_stop_rx(dev);
    sleep(1);
  }
}
typedef enum { /* 2014.8.1 */
	HACKRF_TRANSCEIVER_MODE_OFF = 0,
	HACKRF_TRANSCEIVER_MODE_RECEIVE = 1,
	HACKRF_TRANSCEIVER_MODE_TRANSMIT = 2,
} hackrf_transceiver_mode;
extern int hackrf_set_transceiver_mode(hackrf_device* device, hackrf_transceiver_mode value);
void blink(int times) {
  while (times--) {
    hackrf_set_transceiver_mode(dev, HACKRF_TRANSCEIVER_MODE_RECEIVE);
    sleep(1);
    hackrf_set_transceiver_mode(dev, HACKRF_TRANSCEIVER_MODE_OFF);
    sleep(1);
  }
}
void blinkd(int times) {
  hackrf_hush();
  while (times--) {
    hackrf_set_transceiver_mode(dev, HACKRF_TRANSCEIVER_MODE_TRANSMIT);
    sleep(1);
    hackrf_set_transceiver_mode(dev, HACKRF_TRANSCEIVER_MODE_RECEIVE);
    sleep(1);
    hackrf_set_transceiver_mode(dev, HACKRF_TRANSCEIVER_MODE_OFF); /* 2014.8.1 */
  }
}

/* for (setrate = 1; setrate < 1e9; setrate *= 10)
     printf("Rate: Set: %.0f Got: %ld\n", setrate, check_dsamplerate(setrate)); */
int _sc_ret = 0; /* else stop (??sollte) */
int _count(hackrf_transfer *transfer) {
  printf("%x %d %d %d\n", (unsigned int)transfer->buffer,
	 transfer->buffer_length, transfer->valid_length,
	 hackrf_is_streaming(dev) );
  count += transfer->valid_length/2;
  return _sc_ret;
}
unsigned long int _check_samplerate() {
  #define CheckSecs 4
  count = 0;
/*  res = hackrf_set_sample_rate(dev, rate);
  if (tr(res, "hackrf_set_sample_rate")) return 0;*/
  res = hackrf_start_rx(dev, _count, NULL);
  if (tr(res, "hackrf_start_rx")) return 0;
  sleep(CheckSecs);
  hackrf_stop_rx(dev);
  return count / CheckSecs;
}
unsigned long int check_dsamplerate(double rate) {
  res = hackrf_set_sample_rate(dev, rate);
  if (tr(res, "hackrf_set_sample_rate")) return 0;
  return _check_samplerate();
}
unsigned long int check_fsamplerate(unsigned int rate, unsigned int fract) {
  res = hackrf_set_sample_rate_manual(dev, rate, fract);
  if (tr(res, "hackrf_set_sample_rate_manual")) return 0;
  return _check_samplerate();
}

int _nucb(hackrf_transfer *transfer) {
  count++;
  return 0;
}
unsigned long _nutest(double rate) {
  res = hackrf_set_sample_rate(dev, rate);
  //if (tr(res, "hackrf_set_sample_rate")) return 0;
  res = hackrf_start_rx(dev, _nucb, NULL);
  //if (tr(res, "hackrf_start_rx")) return 0;
  res = hackrf_set_freq(dev, 145000000);
  printf("Rate: %.0f %d", rate, hackrf_is_streaming(dev)); fflush(stdout);
  sleep(5);
  printf(" %d Count: %ld\n", hackrf_is_streaming(dev), count);
  hackrf_stop_rx(dev);
  return count;
}
nutest()
{
  int bw, res;
  bw  = hackrf_compute_baseband_filter_bw(2e6);
  res = hackrf_set_baseband_filter_bandwidth(dev, bw);

  count = 0; _nutest(8e6);
  count = 0; _nutest(10e6);
  count = 0; _nutest(16e6);
  count = 0; _nutest(20e6);
}

int main(int argc, char **argv)
{
  uint32_t setrate;

  if (argc > 1) switch(argv[1][1]) {
    case 'd': ++debug;   break;
    case 'v': ++verbose; break;
  }

  if (!hackrf_begin()) {
    if (verbose) printf("Can't open HackRf\n");
    return -1;
  }
  hackrf_hush();

  nutest();

  hackrf_hush();
  hackrf_end();
  return 0;
}
